package com.hongyegroup.cpt_ewallet.mvvm

import android.annotation.SuppressLint
import android.app.Activity
import android.view.View
import android.widget.TextView
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.SavedStateHandle
import com.bigkoo.pickerview.builder.TimePickerBuilder
import com.guadou.cs_cptservices.binding.BaseDataBindingAdapter
import com.guadou.lib_baselib.base.vm.BaseViewModel
import com.guadou.lib_baselib.ext.checkEmpty
import com.guadou.lib_baselib.utils.CommUtils
import com.hongyegroup.cpt_ewallet.BR
import com.hongyegroup.cpt_ewallet.R
import com.hongyegroup.cpt_ewallet.bean.WalletHistoryBean
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.withContext
import java.util.*
import javax.inject.Inject

@HiltViewModel
class EWalletHistoryViewModel @Inject constructor(
    private val savedStateHandle: SavedStateHandle
) : BaseViewModel() {

    val mKeywordLiveData = MutableLiveData<String>()  //搜索Key值

    var mCurPage = 1
    private var isNeedPlaceHolder = true
    var isNeedCleanAllData = true
    var mDatas = mutableListOf<WalletHistoryBean>()
    val mAdapter by lazy { BaseDataBindingAdapter(R.layout.item_ewallet_history, BR.item, mDatas) }

    private val startCalendar = Calendar.getInstance()
    private val endCalendar = Calendar.getInstance()
    private var mSelectTimeStamp: Long = System.currentTimeMillis()

    init {
        //当前页面时间设置区间
        startCalendar.add(Calendar.YEAR, -3)
    }

    /**
     * 获取兼职工作列表
     */
    fun requestHistoryList(): LiveData<Boolean> {
        val liveData = MutableLiveData<Boolean>()

        launchOnUI {
            //开始Loading
            if (isNeedPlaceHolder) loadStartLoading()
            val result = withContext(Dispatchers.IO) {
                delay(1000)

                val list = mutableListOf<WalletHistoryBean>()
                for (i in 1..10) {
                    list.add(WalletHistoryBean().apply {
                        groupDate = "2022-03-$mCurPage 星期二"
                    })
                }
                return@withContext list
            }

            loadSuccess()
            handleData(result)
            liveData.postValue(true)
        }

        return liveData
    }

    //处理数据-添加或刷新
    @SuppressLint("NotifyDataSetChanged")
    private fun handleData(list: List<WalletHistoryBean>?) {
        if (!list.checkEmpty()) {
            //有数据，判断是刷新还是加载更多的数据
            if (isNeedCleanAllData) {
                //刷新的方式
                mDatas.clear()
                mDatas.addAll(list!!)
                mAdapter.notifyDataSetChanged()
            } else {
                //加载更多
                mDatas.addAll(list!!)
                mAdapter.notifyItemRangeInserted(mDatas.size - list.size, list.size)
                mAdapter.loadMoreModule.loadMoreComplete()
            }
        } else {
            //展示无数据
            if (isNeedCleanAllData && mCurPage == 1) {
                mDatas.clear()
                mAdapter.notifyDataSetChanged()
                //展示无数据的布局
                val emptyView: View = CommUtils.inflate(R.layout.item_empty_layout)
                emptyView.findViewById<TextView>(R.id.tv_empty_msg).text = "你没有任何记录"
                mAdapter.setEmptyView(emptyView)
            } else {
                //如果是加载更多,展示加载完成，没有更多数据了
                mAdapter.loadMoreModule.loadMoreEnd()
            }
        }
        isNeedPlaceHolder = false
        isNeedCleanAllData = false
    }

    /**
     * 选择日期筛选
     */
    fun selectDateMonth(activity: Activity): LiveData<String> {
        val liveData = MutableLiveData<String>()

        //时间选择器
        val pickerBuilder = TimePickerBuilder(activity) { date, _ ->
            mSelectTimeStamp = date.time
            liveData.value = mSelectTimeStamp.toString()
        }
            .isDialog(true)
            .setRangDate(startCalendar, endCalendar)
            .setTitleText("选择月份")
            .setType(booleanArrayOf(true, true, false, false, false, false))

        //默认选中现在时间
        val calendar = Calendar.getInstance()
        calendar.timeInMillis = mSelectTimeStamp
        pickerBuilder.setDate(calendar)

        pickerBuilder.build().show()

        return liveData
    }

}