package com.hongyegroup.cpt_ewallet.ui

import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.TextView
import com.chad.library.adapter.base.listener.OnLoadMoreListener
import com.gavin.com.library.PowerfulStickyDecoration
import com.gavin.com.library.listener.PowerGroupListener
import com.guadou.cs_cptservices.base.activity.YYBaseVDBActivity
import com.guadou.lib_baselib.bean.DataBindingConfig
import com.guadou.lib_baselib.engine.toast
import com.guadou.lib_baselib.ext.dp2px
import com.guadou.lib_baselib.ext.vertical
import com.guadou.lib_baselib.utils.CommUtils
import com.guadou.lib_baselib.utils.KeyboardUtils
import com.hongyegroup.cpt_ewallet.BR
import com.hongyegroup.cpt_ewallet.R
import com.hongyegroup.cpt_ewallet.databinding.ActivityEwalletHistoryBinding
import com.hongyegroup.cpt_ewallet.mvvm.EWalletHistoryViewModel
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshListener
import dagger.hilt.android.AndroidEntryPoint

/**
 * 电子钱包的历史记录列表
 */
@AndroidEntryPoint
class EWalletHistoryActivity : YYBaseVDBActivity<EWalletHistoryViewModel, ActivityEwalletHistoryBinding>(), OnRefreshListener,
    OnLoadMoreListener {

    private var mTopDecoration: PowerfulStickyDecoration? = null
    private val mClickProxy by lazy { ClickProxy() }

    companion object {
        fun startInstance() {
            val context = CommUtils.getContext()
            val intent = Intent(context, EWalletHistoryActivity::class.java)
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            context.startActivity(intent)
        }
    }

    override fun getDataBindingConfig(): DataBindingConfig {
        return DataBindingConfig(R.layout.activity_ewallet_history, BR.viewModel, mViewModel)
            .addBindingParams(BR.click, mClickProxy)
    }

    override fun init(savedInstanceState: Bundle?) {
        setStatusBarBlackText()

        initRV()
        initData()
        initListener()
    }

    private fun initData() {
        mViewModel.requestHistoryList().observe(this) {
            mBinding.refreshLayout.finishRefresh()
            mViewModel.mAdapter.loadMoreModule.isEnableLoadMore = true

            //清楚悬停头缓存
            if (mViewModel.isNeedCleanAllData) {
                mTopDecoration?.clearCache()
            }
        }
    }

    private fun initRV() {

        //悬停头的定义
        val listener: PowerGroupListener = object : PowerGroupListener {
            override fun getGroupName(position: Int): String? {
                //获取组名，用于判断是否是同一组
                return if (mViewModel.mDatas.size > position) {
                    mViewModel.mDatas[position].groupDate
                } else null
            }

            override fun getGroupView(position: Int): View {
                //获取自定定义的组View
                return if (mViewModel.mDatas.size > position) {
                    layoutInflater.inflate(R.layout.item_history_income_header, null, false).apply {
                        findViewById<TextView>(R.id.tv_head_job_date).text = mViewModel.mDatas[position].groupDate
                    }
                } else {
                    View(mActivity).apply { setBackgroundColor(Color.WHITE) }
                }
            }
        }
        mTopDecoration = PowerfulStickyDecoration.Builder
            .init(listener)
            .setGroupHeight(dp2px(40f))
            .setCacheEnable(true)
            .setHeaderCount(0) //是否有头布局->头布局的数量
            .setOnClickListener { _, _ -> //Head点击事件
                toast("查询当前月份的月度清单")
            }
            .build()

        //设置悬停头和Adapter
        mBinding.recyclerView.vertical()
            .apply {
                addItemDecoration(mTopDecoration!!)
                adapter = mViewModel.mAdapter
            }
    }

    private fun initListener() {
        //Adapter的滑动监听，监听加载更多
        mViewModel.mAdapter.loadMoreModule.isEnableLoadMore = false
        mViewModel.mAdapter.loadMoreModule.preLoadNumber = 4
        mViewModel.mAdapter.loadMoreModule.setOnLoadMoreListener(this)

        //刷新控件初始化
        mBinding.refreshLayout.setEnableNestedScroll(true)
        mBinding.refreshLayout.setEnableLoadMore(false)
        mBinding.refreshLayout.setEnableRefresh(true)
        mBinding.refreshLayout.setOnRefreshListener(this)

        //Item点击
        mViewModel.mAdapter.addChildClickViewIds(R.id.ll_content_root)
        mViewModel.mAdapter.setOnItemChildClickListener { _, _, position ->
            val item = mViewModel.mDatas[position]
            toast("去历史详情页面")
        }

        //右上角时间筛选
        mBinding.easyTitle.addRightImg(R.drawable.title_red_date_icon) {
            mViewModel.selectDateMonth(mActivity).observe(this) {
                scrollTopRefresh()
            }
        }
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        mViewModel.isNeedCleanAllData = true
        mViewModel.mAdapter.loadMoreModule.loadMoreComplete()
        mViewModel.mAdapter.loadMoreModule.isEnableLoadMore = false
        mViewModel.mCurPage = 1
        initData()
    }

    override fun onLoadMore() {
        mViewModel.isNeedCleanAllData = false
        mViewModel.mCurPage++
        initData()
    }

    //自动刷新
    private fun scrollTopRefresh() {
        if (mViewModel.mDatas.size > 1) {
            mBinding.recyclerView.scrollToPosition(0)
        }
        mBinding.refreshLayout.autoRefresh()
    }

    override fun startObserve() {

    }

    /**
     * DataBinding事件处理
     */
    inner class ClickProxy {

        //执行搜索
        fun doSearch() {
            KeyboardUtils.hideSoftInput(mActivity)
            scrollTopRefresh()
        }

        //搜索的删除
        fun searchDel() {
            mViewModel.mKeywordLiveData.value = ""
            doSearch()
        }

    }
}